@@ -1,7 +1,7 @@
 package Devel::Backtrace::Point;
 use strict;
 use warnings;
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 use Carp;
 use String::Escape qw(printable);
 
@@ -67,10 +67,22 @@ supports hinthash by using L<UNIVERSAL/can>:
 
 __PACKAGE__->mk_ro_accessors(FIELDS);
 
+=head2 $p->level
+
+This is the level given to new().  It's intended to be the parameter that was
+given to caller().
+
+=cut
+
+__PACKAGE__->mk_ro_accessors('level');
+
 =head2 $p->called_package
 
 This returns the package that $p->subroutine is in.
 
+If $p->subroutine does not contain '::', then '(unknown)' is returned.  This is
+the case if $p->subroutine is '(eval)'.
+
 =cut
 
 sub called_package {
@@ -106,11 +118,24 @@ an array holding the return values of caller().  This array must have either
 three or ten elements (or eleven if hinthash is supported) (see
 L<perlfunc/caller>).
 
+Optional additional parameters:
+
+    -format => 'formatstring',
+    -level => $i
+
+The format string will be used as a default for to_string().
+
+The level should be the parameter that was given to caller() to obtain the
+caller information.
+
 =cut
 
+__PACKAGE__->mk_ro_accessors('_format');
+__PACKAGE__->mk_accessors('_skip');
+
 sub new {
     my $class = shift;
-    my ($caller) = @_;
+    my ($caller, %opts) = @_;
 
     my %data;
 
@@ -128,9 +153,27 @@ sub new {
         croak 'That does not look like the return values of caller.';
     }
 
+    for my $opt (keys %opts) {
+        if ('-format' eq $opt) {
+            $data{'_format'} = $opts{$opt};
+        } elsif ('-level' eq $opt) {
+            $data{'level'} = $opts{$opt};
+        } elsif ('-skip' eq $opt) {
+            $data{'_skip'} = $opts{$opt};
+        } else {
+            croak "Unknown option $opt";
+        }
+    }
+
     return $class->SUPER::new(\%data);
 }
 
+sub _virtlevel {
+    my $this = shift;
+
+    return $this->level - ($this->_skip || 0);
+}
+
 =head2 $tracepoint->to_string()
 
 Returns a string of the form "Blah::subname called from main (foo.pl:17)".
@@ -140,16 +183,75 @@ package C<main> in C<foo.pl> line 17.
 If you print a C<Devel::Backtrace::Point> object or otherwise treat it as a
 string, to_string() will be called automatically due to overloading.
 
+Optional parameters: -format => 'formatstring'
+
+The format string changes the appearance of the return value.  It can contain
+C<%p> (package), C<%c> (called_package), C<%f> (filename), C<%l> (line), C<%s>
+(subroutine), C<%a> (hasargs), C<%e> (evaltext), C<%r> (is_require), C<%h>
+(hints), C<%b> (bitmask), C<%i> (level), C<%I> (level, see below).
+
+The difference between C<%i> and C<%I> is that the former is the argument to
+caller() while the latter is actually the index in $backtrace->points().  C<%i>
+and C<%I> are different if C<-start>, skipme() or skipmysubs() is used in
+L<Devel::Backtrace>.
+
+If no format string is given, the one passed to C<new> will be used.  If none
+was given to C<new>, the format string defaults to 'default', which is an
+abbreviation for C<%s called from %p (%f:%l)>.
+
+Format strings have been added in Devel-Backtrace-0.10.
+
 =cut
 
-sub to_string {
-    my $this = shift;
+my %formats = (
+    'default' => '%s called from %p (%f:%l)',
+);
 
-    return $this->subroutine
-      . ' called from '
-      . $this->package . ' ('
-      . $this->filename . ':'
-      . $this->line . ')';
+my %percent = (
+    'p' => 'package',
+    'c' => 'called_package',
+    'f' => 'filename',
+    'l' => 'line',
+    's' => 'subroutine',
+    'a' => 'hasargs',
+    'w' => 'wantarray',
+    'e' => 'evaltext',
+    'r' => 'is_require',
+    'h' => 'hints',
+    'b' => 'bitmask',
+    'i' => 'level',
+    'I' => '_virtlevel',
+);
+
+sub to_string {
+    my ($this, @opts) = @_;
+
+    my %opts;
+    if (defined $opts[0]) { # check that we are not called as stringification
+        %opts = @opts;
+    }
+
+    my $format = $this->_format();
+
+    for my $opt (keys %opts) {
+        if ($opt eq '-format') {
+            $format = $opts{$opt};
+        } else {
+            croak "Unknown option $opt";
+        }
+    }
+
+    $format = 'default' unless defined $format;
+    $format = $formats{$format} if exists $formats{$format};
+
+    my $result = $format;
+    $result =~ s{%(\S)} {
+        my $percent = $percent{$1} or croak "Unknown symbol %$1\n";
+        my $val = $this->$percent();
+        defined($val) ? printable($val) : 'undef';
+    }ge;
+
+    return $result;
 }
 
 =head2 $tracepoint->to_long_string()
@@ -176,12 +278,13 @@ hinthash is not included in the output, as it is a hash.
 
 sub to_long_string {
     my $this = shift;
-    return join '', grep {
-        ! /^_/
-    } map {
+    return join '',
+    map {
 	"$_: " .
 	(defined ($this->{$_}) ? printable($this->{$_}) : 'undef')
 	. "\n"
+    } grep {
+        ! /^_/ && 'hinthash' ne $_
     } FIELDS;
 }
 
